@@ -365,6 +365,10 @@ func DecryptFileInfo(fi FileInfo, folderKey *[keySize]byte) (FileInfo, error) {
 	if err := proto.Unmarshal(dec, &decFI); err != nil {
 		return FileInfo{}, err
 	}
+
+	// Preserve sequence, which is legitimately controlled by the untrusted device
+	decFI.Sequence = fi.Sequence
+
 	return decFI, nil
 }
 
@@ -543,7 +547,7 @@ func slashify(s string) string {
 // deslashify removes slashes and encrypted file extensions from the string.
 // This is the inverse of slashify().
 func deslashify(s string) (string, error) {
-	if len(s) == 0 || !strings.HasPrefix(s[1:], encryptedDirExtension) {
+	if s == "" || !strings.HasPrefix(s[1:], encryptedDirExtension) {
 		return "", fmt.Errorf("invalid encrypted path: %q", s)
 	}
 	s = s[:1] + s[1+len(encryptedDirExtension):]
@@ -558,8 +562,8 @@ func (r rawResponse) Data() []byte {
 	return r.data
 }
 
-func (r rawResponse) Close() {}
-func (r rawResponse) Wait()  {}
+func (rawResponse) Close() {}
+func (rawResponse) Wait()  {}
 
 // IsEncryptedParent returns true if the path points at a parent directory of
 // encrypted data, i.e. is not a "real" directory. This is determined by
@@ -571,7 +575,7 @@ func IsEncryptedParent(pathComponents []string) bool {
 	} else if l == 0 {
 		return false
 	}
-	if len(pathComponents[0]) == 0 {
+	if pathComponents[0] == "" {
 		return false
 	}
 	if pathComponents[0][1:] != encryptedDirExtension {
